// 3-MAY-2014 -Added Backlight Toggle
//            -Added Interrupt for Pushing a "1" to vera by using a pushbutton on interrupt 1 (pin 3)
// Humidity and Temperature Sensor with LCD Display
// Retrieves Temperature, Humidity, Weather (text) and today's Hi/Low from Vera and displays it on an LCD alongside with Room Temperature and Humidity.
#include <Sleep_n0m1.h>
#include <SPI.h>
#include <EEPROM.h>  
#include <RF24.h>
#include <Sensor.h>  
#include <DHT.h> 
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>
//
LiquidCrystal_I2C lcd(0x27,20,4);  // set the LCD address to 0x20 for a 16 chars and 2 line display
//
#define CHILD_ID_HUM 0
#define CHILD_ID_TEMP 1
#define HUMIDITY_SENSOR_DIGITAL_PIN 4
unsigned long SLEEP_TIME = 5; // Sleep time between reads (in seconds)
//
Sensor gw;
DHT dht;
Sleep sleep;
float lastTemp;
float lastHum;
float temperature;
float humidity;
float temp;
boolean metric = false;
int OutdoorHumidity;
int OutdoorTemp;
int TodayLow;
int TodayHigh;
String TodayWeather= "No Weather Reported"; //*****
int ledStatus = 1;// to toggle LCD backlight led
int counter = 30;
int MessageToggle = 0;
int pin = 3;
volatile int state = LOW;
//
void setup()  
{ 
  Serial.begin(115200);
  pinMode(pin, OUTPUT);//interrupt on pin 3
  attachInterrupt(1, PushButton, CHANGE);//interrupt pin 3 is interrupt 1
  gw.begin();
  dht.setup(HUMIDITY_SENSOR_DIGITAL_PIN); 
  gw.sendSketchInfo("LCD Multi", "1.0");
  gw.sendSensorPresentation(CHILD_ID_HUM, S_HUM);
  gw.sendSensorPresentation(CHILD_ID_TEMP, S_TEMP);
  gw.getStatus(CHILD_ID_HUM, V_VAR1);//current exterior humidity
  gw.getStatus(CHILD_ID_HUM, V_VAR2);//Backlight Toggle 0=off 1==ON
  gw.getStatus(CHILD_ID_HUM, V_VAR3);//PushButton toggle
  gw.getStatus(CHILD_ID_TEMP,V_VAR1);//current exterior temperature
  gw.getStatus(CHILD_ID_TEMP,V_VAR2);//today's high temperature
  gw.getStatus(CHILD_ID_TEMP,V_VAR3);//today's low temperature
  gw.getStatus(CHILD_ID_TEMP,V_VAR4);//todays weather
  //
  gw.sendVariable(CHILD_ID_HUM, V_VAR3, 0);//reset PushButton toggle
  //
  lcd.init(); // initialize the lcd 
  // Print a message to the LCD.
  lcd.backlight();
  //lcd.setBacklight(LOW);******** come back to this
  lcd.setCursor(0,0);
  lcd.print("   Place Your   ");
  lcd.setCursor(0,1);
  lcd.print("    Startup     ");
  delay(2000);
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("    Message     ");
  lcd.setCursor(0,1);
  lcd.print("     Here       ");
  delay(2000);
}
//
void loop()      
{  
  delay(dht.getMinimumSamplingPeriod());
  if (counter > 29){
    gw.getStatus(CHILD_ID_HUM, V_VAR1);
    HumStatus(gw.getMessage());
    gw.getStatus(CHILD_ID_HUM, V_VAR2);
    ledToggle(gw.getMessage());
    gw.getStatus(CHILD_ID_TEMP, V_VAR1);
    TempStatus(gw.getMessage());
    gw.getStatus(CHILD_ID_TEMP, V_VAR2);
    LowStatus(gw.getMessage());
    gw.getStatus(CHILD_ID_TEMP, V_VAR3);
    HighStatus(gw.getMessage());
    gw.getStatus(CHILD_ID_TEMP, V_VAR4);
    WeatherStatus(gw.getMessage());
    Serial.println(TodayWeather);
    counter = 0;
    if (ledStatus == 1) {//checking to toggle LCD's internal backlight LED
       lcd.backlight();
    }
    else{
       lcd.noBacklight();
    }
  }
  temperature = dht.getTemperature();
  temp = dht.toFahrenheit(temperature);
  if (isnan(temperature)) {
      Serial.println("Failed reading temperature from DHT");
  } else if (temperature != lastTemp) {
    lastTemp = temperature;
     if (!metric) {
      temperature = dht.toFahrenheit(temperature);
    }
    gw.sendVariable(CHILD_ID_TEMP, V_TEMP, temperature, 1);
      Serial.print("T: ");
      Serial.println(temperature);
  }
  humidity = dht.getHumidity();
  if (isnan(humidity)) {
      Serial.println("Failed reading humidity from DHT");
  } else if (humidity != lastHum) {
      lastHum = humidity;
      gw.sendVariable(CHILD_ID_HUM, V_HUM, humidity, 1);
      Serial.print("H: ");
      Serial.println(humidity);
  }
  // Power down the radio.  Note that the radio will get powered back up
  // on the next write() call.
  delay(1000); //delay to allow serial to fully print before sleep
  gw.powerDown();
  sleep.pwrDownMode(); //set sleep mode
  sleep.sleepDelay(SLEEP_TIME * 1000); //sleep for: sleepTime
  //
  //
    if (MessageToggle == 1){
    gw.sendVariable(CHILD_ID_HUM, V_VAR3, 1);
    AirConMessage();
    delay(6000);
    MessageToggle=0;
  }
  updateOutsideTemp();
  updateOutsideHumid();
  delay(3000);
  updateLow();
  updateHigh();
  delay(3000);
  updateWeather();
  delay(3000);
  //lcd.noBacklight();
  //delay(3000);
  //lcd.backlight();
  updateTemp();
  updateHumid();
  counter = (counter + 1);
}
void updateTemp(){
  lcd.setCursor(0,0);
  lcd.print("Temperature: ");
  lcd.print(round(temp));
  lcd.print("F ");
}
void updateLow(){
  lcd.setCursor(0,0);
  lcd.print("Today's Low: ");
  lcd.print(round(TodayLow));
  lcd.print("F ");
}
void updateHigh(){
  lcd.setCursor(0,1);
  lcd.print("       High: ");
  lcd.print(round(TodayHigh));
  lcd.print("F ");
}
void updateHumid(){
  lcd.setCursor(0,1);
  lcd.print("Humidity:    ");
  lcd.print(round(humidity));
  lcd.print("% ");
}
void updateOutsideTemp(){
  lcd.setCursor(0,0);
  lcd.print("Outside Temp:");
  lcd.print(OutdoorTemp);
  lcd.print("F ");
}
void updateOutsideHumid(){
  lcd.setCursor(0,1);
  lcd.print("Humidity:    ");
  lcd.print(OutdoorHumidity);
  lcd.print("% ");
}
void updateWeather(){
  lcd.setCursor(0,0);
  lcd.print("Today's Weather ");
  lcd.setCursor(0,1);
  lcd.print("                ");//clear the line
  delay(25);
  lcd.setCursor(0,1);
  lcd.print(TodayWeather);
}
void AirConMessage(){
  if (ledStatus == 1){
    lcd.noBacklight();
    for (int i=0; i < 10; i++){
    lcd.backlight();
    delay(100);
    lcd.noBacklight();
    delay(100);
    lcd.backlight();
    }
  lcd.setCursor(0,0);
  lcd.print(" AC PowerBoost  ");
  lcd.setCursor(0,1);
  lcd.print("***ACTIVATED****");
  }
}
void HumStatus(message_s message){
  if (message.header.type==V_VAR1) {
     OutdoorHumidity = atoi(message.data);
     Serial.print("Outdoor Humidity:");
     Serial.println(OutdoorHumidity);
   }
}
void ledToggle(message_s message){
  if (message.header.type==V_VAR2) {
    ledStatus = atoi (message.data);
  
   }
}
void TempStatus(message_s message){
  if (message.header.type==V_VAR1) {
     OutdoorTemp = atoi(message.data);
   }
}
void LowStatus(message_s message){
  if (message.header.type==V_VAR2) {
     TodayLow = atoi(message.data);
   }
}
void HighStatus(message_s message){
  if (message.header.type==V_VAR3) {
     TodayHigh = atoi(message.data);
   }
}
void WeatherStatus(message_s message){
  if (message.header.type==V_VAR4) {
     String Weather = String(message.data);
     int locator = Weather.indexOf("@");
     Weather = Weather.substring(0, locator);
     TodayWeather= Weather;
   }
}
void PushButton(){
  //gw.sendVariable(CHILD_ID_HUM, V_VAR3, 1);
  MessageToggle = 1;
  Serial.println("INTERRUPT");
}





